#!/usr/bin/perl
## multiple_hash_subroutine_example.pl

## NOTE: You won't have to use this approach once we discuss references
## since you will be able to pass arrays and hashes without flattening them
## and requiring reconstructing the structures

use strict;
use warnings;

## outside hashes (to be populated with data from the subroutine)
my %outside_hash1;
my %outside_hash2;

## when we call create_hashes(), we know the first returned value is
## the size of the first hash, with the remaining elements being
## the contents of the first and second hashes, which we can then
## assign to an array
my ($hash1_size, @hashes_data) = create_hashes();

## now we need to rebuild the hashes
## we know that the first $hash1_size key/value pairs belong to the first
## hash so we can remove them from @hashes_data and insert them into
## %outside_hash1
for (my $i = 0; $i < $hash1_size; $i++) {

  ## hashes are flattened as a with keys followed by values
  ## (key1, value1, key2, value2, key3, value3, ...)

  ## remove the key from @hashes_data
  my $key = shift @hashes_data;

  ## remove the value from @hashes_data
  my $value = shift @hashes_data;

  ## populate %outside_hash1
  $outside_hash1{$key} = $value;
}

## since we removed all the key/value pairs that belong to %outside_hash1
## from @hashes_data, we know that the remaining data belongs to
## %outside_hash2

%outside_hash2 = @hashes_data;

## print the contents of hashes
print "Contents of \%outiside_hash1\n";
foreach my $key (keys %outside_hash1) {
  print "$key\t-\t$outside_hash1{$key}\n";
}
print "Contents of \%outiside_hash2\n";
foreach my $key (keys %outside_hash2) {
  print "$key\t-\t$outside_hash2{$key}\n";
}


## subroutines ##

## this subroutine will create two hashes internally and return them
## as a flattened list
sub create_hashes {

  ## inside hashes (to be returned by subroutine)
  my %inside_hash1 = (
    "one"       => 1,
    "two"       => 2,
    "three"     => 3
  );
  my %inside_hash2 = (
    "four"      => 4,
    "five"      => 5
  );

  ## get the number of keys inside the first hash
  ## calling keys %inside_hash1 returns an array of keys for %inside_hash1
  ## which we can then call scalar on to get the size
  my $inside_hash1_num_of_keys = scalar(keys %inside_hash1);

  ## return the hashes -- this will flatten both hashes
  ## will create a list containing the following:
  ## ("one", 1, "two", 2, "three", 3, "four", 4, "five", 5)
  ## since both hashes have become a single list, we need also return
  ## the size of the first hash so that we know how many keys/values to
  ## remove from the list and rebuild our hash
  return ($inside_hash1_num_of_keys, %inside_hash1, %inside_hash2);
}
